#!/usr/bin/guile \
-e (@\ (gnunet-download)\ main) -L . -s
!#
;;;; Copyright © 2015 Rémi Delrue <asgeir@free.fr>
;;;; 
;;;; This program is free software: you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation, either version 3 of the License, or
;;;; (at your option) any later version.
;;;; 
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;; 
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.

(define-module (gnunet-download)
  #:use-module (ice-9 match)
  #:use-module (system foreign)
  #:use-module (gnu gnunet common)
  #:use-module (gnu gnunet container metadata)
  #:use-module (gnu gnunet fs)
  #:use-module (gnu gnunet fs uri)
  #:use-module (gnu gnunet fs progress-info)
  #:use-module (gnu gnunet configuration)
  #:use-module (gnu gnunet scheduler)
  #:export     (main))

(define config-file "~/.gnunet/gnunet.conf")

(define *fs-handle* %null-pointer)
(define *dl-handle* %null-pointer)
(define *stderr* (current-error-port))
(define *count* 1)


(define (shutdown-task _)
  (simple-format *stderr* "scheduler run: timeout\n")
  (force-output *stderr*)
  (display "Shutdown\n")
  (simple-format *stderr* "shutdown-task: stopping dl ~a\n" *dl-handle*)
  (stop-download *dl-handle*)
  (simple-format *stderr* "shutdown-task: stopped dl\n"))

(define (progress-cb %info)
  (simple-format *stderr* "scheduler run: progress-cb ~a ~a\n"
		 *count* (progress-info-status %info))
  (force-output *stderr*)
  (set! *count* (1+ *count*))
  (let ((status (progress-info-status %info)))
    (cond ((equal? status '(#:download #:start))
	   (match (parse-c-progress-info %info)
	     (((%context cctx pctx sctx %uri %filename . _) . _)
	      (simple-format #t "Starting download `~a'.\n"
			     (pointer->string %filename)))))
	  ((equal? status '(#:download #:completed))
	   (match (parse-c-progress-info %info)
	     (((%context cctx pctx sctx %uri %filename . _) . _)
	      (simple-format #t "Downloading `~a' done.\n"
			     (pointer->string %filename))))))))

(define (main args)
  (let ((config (load-configuration config-file)))
    (define (first-task _)
      (simple-format *stderr* "scheduler run: first-task\n")
      (force-output *stderr*)
      (match args
	((binary-name output-filename uri-string)
	 (set! *fs-handle* (open-filesharing-service config binary-name
						     progress-cb))
	 (let ((uri (parse-uri uri-string)))
	   (set! *dl-handle* (start-download *fs-handle* uri output-filename))
	   ;; add a timeout in 5 seconds
	   (simple-format *stderr* "scheduler add: timeout\n")
	   (force-output *stderr*)
	   (add-task! shutdown-task #:delay (* 5 1000 1000))))))
    (call-with-scheduler config first-task)))
